////////////////////////////////////////////////////////////////////// 
// libsrc/FilteredPedigree.cpp 
// (c) 2000-2004 Goncalo Abecasis, 2002-2004 Jan Wigginton
// 
// This file is distributed as part of the PEDSTATS source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile PEDSTATS.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Tuesday February 10, 2004
// 
 
#include "FilteredPedigree.h"
#include "Constant.h"
#include "Error.h"
#include "Sort.h"

void FilteredPedigree::InitializeFilter()
   {
   filtered.Dimension(count);
   filtered.Zero();
   }

void FilteredPedigree::ExtractFamily(int id, FilteredPedigree & single_fam_ped)
   {
   Pedigree::ExtractFamily(id, (Pedigree &) single_fam_ped);

   single_fam_ped.InitializeFilter();
   for (int i = families[id]->first; i <= families[id]->last; i++)
      single_fam_ped.filtered[i - families[id]->first] = filtered[i];
   }

void FilteredPedigree::ExtractOnAffection(int a, FilteredPedigree & new_ped, int target_status)
   {
   Pedigree::ExtractOnAffection(a, (Pedigree &) new_ped, target_status);

   new_ped.InitializeFilter();
   new_ped.Filter(filtered);
   }

// Merges additional filter with current filtering rules for pedigree
void FilteredPedigree::Filter(IntArray & filter)
   {
   if (filter.Length() != count)
      error("FilteredPedigree:Size of pedigree filter doesn't match number of persons in pedigree");

   filtered.Dimension(count);
   for (int i = 0; i < count; i++)
         filtered[i] |= filter[i];
   }

void FilteredPedigree::Sort()
   {
   Pedigree::Sort();
   InitializeFilter();
   }
 
