////////////////////////////////////////////////////////////////////// 
// pedstats/pedstats_pdf.h 
// (c) 2000-2004 Goncalo Abecasis, 2002-2004 Jan Wigginton
// 
// This file is distributed as part of the PEDSTATS source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile PEDSTATS.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Tuesday February 10, 2004
// 
 
#ifndef __PEDSTATS_PDF_H__
#define __PEDSTATS_PDF_H__

#include "PDFhistogram.h"
#include "PDFlinechart.h"
#include "PDFgrid.h"
#include "PedigreePairs.h"

// Code for main graph pages
//
void GraphAffectionStatistics(PDF &pdf);
void GraphAffection(PDF & pdf, int a);

void GraphFamilyStatistics(PDF & pdf);

void GraphMarkerStatistics(PDF & pdf);
void GraphMarker(PDF & pdf, int m);

void GraphCovariateStatistics(PDF & pdf);
//void GraphCovariate(PDF & pdf, int c);

void GraphTraitStatistics(PDF & pdf);
void GraphTrait(PDF & pdf, int t, bool is_covariate);

// Code for single graphs
//
void GraphVarOverall(PDF & pdf, PDFHistogram & chart, Vector & values);
void GraphVarBySex(PDF & pdf, PDFHistogram & chart, Vector & v_female,
     Vector & v_male, bool do_subtitle);
void GraphVarByFounder(PDF & pdf, PDFHistogram & chart, Vector & v_founder,
     Vector & v_nonfounder, bool do_subtitle);

// Code for single (pair) graphs
//
void   GraphPairAffection(PDF & pdf, PDFHistogram & chart, SimplePairList & pair_list,
       bool pairs_page, int index, const char * alt_title = "");
int    GetPairAffectionData(SimplePairList & pair_list, PDFHistogram & chart, int index);

void   GraphVarByPairs(PDF & pdf, bool is_covariate, SimplePairList & pairs, bool prs_page,
       bool is_mirrored, const char * alt_title = "");
double GetPairData(SimplePairList & pair_list, PDFLineChart & chart,
       int & sib_pairs, bool is_covariate = false, bool is_mirrored = true);

// Code for all pair pages (--pairs)
//
void GraphVarForAllPairs(PDF & pdf, int filter_var, bool is_covariate);
void GraphAffectionForAllPairs(PDF & pdf, int filter_var);

// Code for gender-specific pair pages (--bySex)
//
void DoAffectionPairsByGender(PDF & pdf, PDFHistogram & chart, SimplePairList & pair_list,
     bool is_mirrored);
void DoVarPairsByGender(PDF & pdf, bool is_covariate, SimplePairList & pair_list,
     bool is_mirrored);
void GetGenderPlotTitle(SimplePairList & pairs, String & alt_title);

// Code for HWE test pages (--checkAll, --foundersHWE)
//
void GraphHardyWeinberg(PDF & pdf, Matrix & observed, Matrix & expected,
     const char *marker_name, StringArray & allele_names, double chisq,
     double pvalue, const char * founder_label, const char * globbed_alleles);
void MakeSignificanceKey(PDF & pdf, double p_value);
void DrawRedCross(PDF & pdf, double x, double y, double width, double height);
void DrawGreenCheck(PDF & pdf, double x, double y, double width, double height, double ratio = 1.0);
void DrawGreenChecks(PDF & pdf, double x, double y, double width, double height);

// Auxilliary graphing functions
//
void MakePageHeader(PDF & pdf, const char *title, const char * sub_title);
void AddPageFootNote(PDF & pdf, double x_pct_offset, const char * note, double y_pct_offset = 0.04);

void DrawSixth(PDF & pdf, PDFChartBasics * chart, int & pr_graph);
void DrawQuadrant(PDF & pdf, PDFChartBasics * chart, int & pr_gender_graph);


#endif



 
